#define MIN_AMS 207
#define OPTIMIZE_ROM_CALLS				// Use Speed Optimization
#define NO_EXIT_SUPPORT						//no need for exit or atexit
#define SAVE_SCREEN

#define fatal(s) {ST_showHelp(s);return;}

#include <tigcclib.h>          //Include All Header Files
#include "h220xTSR.h"					//Kevin Kofler's hw2 ams2 tsr support
#include "array.h"					 //The pre-compiled EVENT HOOK
#include "pics.h"

short _ti89;               // Produce .89Z File



//Your event Hook's name; must be 12 characters long and be preceded by evHk
#define HOOK_NAME "evHkcoolClck"

// Main Function
void _main(void)
{
	unsigned long 	tsrSize;			 //Size of the Event Program
	volatile HANDLE tsrHandle;		//Handle of allocated mem for the Hook
	void 						*pTsrMem;		 //a pointer to the actual mem for the Hook
	void (*HookMain)(void*); 		//a pointer to the event hook's Install_TSR function
	void *pHook=EV_hook-16;		//points to where evHk should be in an installed hook
		const HARDWARE_PARM_BLOCK *hpb = FL_getHardwareParmBlock ();
		unsigned long LCDHeight, LCDWidth;

		//Logo
		GrayOn();
		OutputBackground();
		ngetchx();
		GrayOff();
		clrscr();
		
		DlgMessage ("BSoft coolClock 1.1", "BSoft coolClock copyright 2002 BSoft LLC. Liscenced under the terms of the GNU GPL. See COPYING.TXT on desktop for details.", BT_NONE, BT_OK);
		if (!(hpb->len > OFFSETOF (HARDWARE_PARM_BLOCK, hardwareRevision)))
		{
				DlgMessage ("BSoft coolClock 1.1", "BSoft coolClock can not install because your calculator does not contain the required clock hardware. A hardware-revision-two calculator is required.", BT_NONE, BT_OK);
				return;
		}	
/////////////////////////////////////////
// Program independent error checking	//
///////////////////////////////////////

	//see if the hook has already been installed
		if(EV_hook)
			while(pHook)
			{
			//Already installed
			if(!memcmp(pHook,HOOK_NAME,12))
			{
				DlgMessage ("BSoft coolClock 1.1", "Coolclock is already installed. Use uninevhk to unistall.", BT_NONE, BT_OK);
				return;
			}
			
			//the installed hook does not follow the convention so I cannot
			//check for a previous installation
				if(memcmp("evHk",pHook,4))
					break;
			
			//Get the address for the next hook...
				(unsigned long)pHook=*(unsigned long*)(pHook+12)-16;
			}
	

///////////////////////////////////
// End Error checking/handling	//
/////////////////////////////////

	//sort of like a call to a permanent version of enter_ghost_space();
		if(!h220xTSR())
		{
			DlgMessage ("BSoft coolClock 1.1", "Not enoguh memory.", BT_NONE, BT_OK);
			return;
		}
			
	//allocate RAM for the Event Hook
		tsrSize=sizeof(tsr)-2;
		tsrHandle=HeapAllocHigh(tsrSize+16);
	
	//Exit If there is an error getting a handle of course
		if (!tsrHandle)
		{
			DlgMessage ("BSoft coolClock 1.1", "Not enoguh memory.", BT_NONE, BT_OK);
			return;
		}


	//get a pointer to the actual mem
		pTsrMem=HeapDeref(tsrHandle);
		pTsrMem+=0x40000;	//a necessary evil ;) bypasses some protections...
	
	// the signature
	// the string MUST ALWAYS be exactly 12 characters long!
	//Just for the record, the null terminating character is overwritten
	//when saving the old tsr hook's address.
		strcpy(pTsrMem,HOOK_NAME);
			pTsrMem+=12;

	//Save the Old tsr hook's address for the uninstall prog
		*((unsigned long*)pTsrMem)=(unsigned long)EV_hook;
			pTsrMem+=4;

/*
This is outdated, because of the new startup code in the actual Event Hook
This copies the op-code for jmp EventHandler to the correct spot in the tsr.
EV_hook will be set to this address to install the hook
*/
	//0x4EF9 is the op-code for an absolute jump (2 bytes)
		//*(unsigned int*)pTsrMem=0x4EF9;
										//2 + 4 
			//pTsrMem+=6;	 //(skipping the 4 byte area for the address of the 
									//Event handler) It will be filled in by the TSR's install
								 //function.

	//Copy the hook to a permanent location (locked handle) in the RAM
	//skipping the length bytes
		memcpy(pTsrMem,tsr+2,tsrSize);

	//Relocate the tsr
		EX_patch(pTsrMem,pTsrMem+tsrSize-1);

//Call the main function of the tsr and pass a pointer
//to the pointer to the old tsr...
//HookMain is actually a variable that points to the 
//Hook's install function defined as _main(void *pOldHandler)

	//Get the address of the tsr's install function
		HookMain=pTsrMem;
		
	 //Pass a pointer to the pointer to the old event hook which 
	//is located 4 bytes before the tsr's install function
	//4=sizeof the pointer
		HookMain(pTsrMem-4);

//finally set the new tsr as the first tsr to be called
//This may look wierd because pTsrMem was used to call the install function
//but it is correct because the tsr's install function patches itself
//so that it will jump to the event handler instead of executing Install_TSR
		EV_hook=pTsrMem;

		DlgMessage ("BSoft coolClock 1.1", "coolClock successfully installed (it will take a few seconds for the clock to appear). Use uninevhk to uninstall. You may safely delete the installer program.", BT_NONE, BT_OK);
}

void OutputBackground(void) {
    unsigned char len,val,counter = 0;
    unsigned char *source;
    unsigned char *dest;

    unsigned short size;
    unsigned short i;

    memset(GetPlane(LIGHT_PLANE),0,LCD_SIZE);
    memset(GetPlane(DARK_PLANE), 0,LCD_SIZE);

    for (i=0;i<2;i++) {
        if (i==0) {
            source = plane1;
            dest   = GetPlane(LIGHT_PLANE);
        }
        else {
            source = plane2;
            dest   = GetPlane(DARK_PLANE);
        }

        size = 2000; // 160*100/8

        
           while (size) {
            if (*source == 0x91) {
                source++;
                val = *source++;
                len = *source++;
                while (len) {
                    *dest++ = val;
                    len--;
                    size--;
                    counter++;
                    if (counter == 20) {
                        dest+=10;
                        counter=0;
                    }
               }
           }
           else {
               *dest++ = *source++;
               size--;
               counter++;
               if (counter == 20) {
                   dest+=10;
                   counter=0;
               }
           }
       }
   }
}